#!/usr/bin/perl -w
#Author: liangf

use strict;
use warnings;
use Getopt::Long;
use File::Basename;

my %opts=qw();
GetOptions(\%opts,"perlDir:s","reads1:s","reads2:s","gene:s","rawRef:s","convertRef:s","pathToPython:s","pathToHISAT2:s","pathToSamtools:s","phred64!","specBuild:s","specHisat2:s","outDir:s","help|h!");


my @usage=qq(
============================= Bisulfite Sequence RNA data analysis pipeline (BS-RNA) ============================

                                             Version 1.0

Usage: BS-RNA_v1.0 [Options]

Options: 
  --perlDir          Path    Directory of the perl scripts
  --reads1           File    Input T-rich reads file 
  --reads2           File    Input A-rich reads file
  --gene             File    Supply bsRNA with a set of gene model annotations, a GTF formatted file
  --rawRef           Path    Directory of raw reference genome sequences
  --convertRef       Path    Directory of converted reference genome sequences
  --pathToPython     Path    Full path </../../> to the Python installation on your system
                             If not specified it is assumed that Python is in the PATH
  --pathToHISAT2     Path    Full path </../../> to the HISAT2 installation on your system
                             If not specified it is assumed that HISAT2 is in the PATH
  --pathToSAMtools   Path    Full path </../../> to the SAMtools installation on your system
                             If not specified it is assumed that SAMtools is in the PATH
  --phred64                  Qualities are ASCII chars equal to the Phred quality plus 64
                             "off" means Qualities are ASCII chars equal to the Phred quality plus 33
                             Default: off
  --specBuild        File    Configure file for hisat2-build indexer
  --specHisat2       File    Configure file for hisat2
  --ourDir           Path    Result output directory
  --h or help                Display this message
);


my $perldir;
my $g;
my $refin;
my $refout;
my $pp;
my $ph;
my $ps;
my $q;
my $opt1;
my $opt2;
my $outdir;

if ((scalar(keys %opts) == 0) or (exists($opts{"help"})) or (exists($opts{"h"}))) {
	print join("\n",@usage)."\n";
	exit;
}

if (exists $opts{"perlDir"}) {
	$perldir = $opts{"perlDir"};
	$perldir =~ s/\/$//;
} else {
	die "Please input the directory of the perl scripts!!!\n";
}

if (exists $opts{"gene"} and $opts{"gene"} =~ /\.gtf$/i) {
	$g = $opts{"gene"};
} else {
	die "Please input a file with a set of gene model annotations, a GTF formatted file with the name ended with .gtf!!!\n";
}

if (exists $opts{'rawRef'}) {
	$refin = $opts{'rawRef'};
	$refin =~ s/\/$//;
} else {
	die "Please input the directory of raw genome reference sequences!!!\n";
}

if (exists $opts{"pathToPython"}) {
	$pp = $opts{"pathToPython"};
	$pp =~ s/\/$//;
}

if (exists $opts{"pathToHISAT2"}) {
	$ph = $opts{"pathToHISAT2"};
	$ph =~ s/\/$//;
}

if (exists $opts{"pathToSamtools"}) {
	$ps = $opts{"pathToSamtools"};
	$ps =~ s/\/$//;
}

if (exists $opts{'phred64'}) {
	$q = 64;
} else {
	$q = 33;
}

if (exists $opts{"outDir"}) {
	$outdir = $opts{"outDir"};
	$outdir =~ s/\/$//;
} else {
	die "Please input the directory of output result!!!\n";
}

if (exists $opts{'convertRef'}) {
	$refout = $opts{'convertRef'};
	$refout =~ s/\/$//;
} else {
	$refout = "$outdir/ref_C-T_G-A";
	system "mkdir $refout" unless -s "$refout";
}

if (exists $opts{'specBuild'}) {
	$opt1 ="";
	open IN, "$opts{'specBuild'}";
	while (my $aa = <IN>) {
		$aa =~ s/\s+$//;
		$aa =~ s/^\s+//;
		next if $aa =~ /^\#/;
		next unless $aa =~ /\-/;
		$opt1 = $opt1." ".$aa;
	}
	close IN;
} else {
	$opt1 = "";
}

if (exists $opts{'specHisat2'}) {
	$opt2 ="";
	open IN, "$opts{'specHisat2'}";
	while (my $aa = <IN>) {
		$aa =~ s/\s+$//;
		$aa =~ s/^\s+//;
		next if $aa =~ /^\#/;
		next unless $aa =~ /\-/;
		$opt2 = $opt2." ".$aa;
	}
	close IN;
} else {
	$opt2 = "";
}


system "mkdir $outdir/ProcessedReads" unless -s "$outdir/ProcessedReads";
system "mkdir $outdir/Map" unless -s "$outdir/Map";
system "mkdir $outdir/Filter" unless -s "$outdir/Filter";
system "mkdir $outdir/Level" unless -s "$outdir/Level";

#########################################################################################################

unless (exists $opts{'convertRef'}) {
	
	system("date");
	system("echo refAll_C-T_G-A");
	system("perl $perldir/refAll_C-T_G-A.pl $refin $refout");
	
	system("date");
	system("echo hisat2-build");
	if (defined $ph) {
		system("$ph/hisat2-build$opt1 $refout/ref_all_C-T_G-A.fa $refout/ref_all_C-T_G-A");
	} else {
		system("hisat2-build$opt1 $refout/ref_all_C-T_G-A.fa $refout/ref_all_C-T_G-A");
	}
	
	system("date");
	system("echo samtools faidx");
	if (defined $ps) {
		system("$ps/samtools faidx $refout/ref.fa");
	} else {
		system("samtools faidx $refout/ref.fa");
	}
}

system("date");
system("echo double_chr_gtf");
system("perl $perldir/double_chr_gtf.pl -g $g -o $outdir/Map/doubled_gene.gtf");
	
system("date");
system("echo extract_splice_sites");
if (defined $pp) {
	if (defined $ph) {
		system("$pp/python $ph/extract_splice_sites.py $outdir/Map/doubled_gene.gtf >$outdir/Map/splicesites.txt");
	} else {
		system("$pp/python extract_splice_sites.py $outdir/Map/doubled_gene.gtf >$outdir/Map/splicesites.txt");
	}
} else {
	if (defined $ph) {
		system("python $ph/extract_splice_sites.py $outdir/Map/doubled_gene.gtf >$outdir/Map/splicesites.txt");
	} else {
		system("python extract_splice_sites.py $outdir/Map/doubled_gene.gtf >$outdir/Map/splicesites.txt");
	}
}

#############################################################################################################
my ($r1, $r2, $s1, $s2, @arr, @brr, $reads1, $reads2);

if (exists $opts{"reads1"} and exists $opts{"reads2"}) {
	
	$r1 = $opts{"reads1"};
	$r2 = $opts{"reads2"};
	$s1 = basename($r1);
	$s2 = basename($r2);
	@arr = split(/\./, $s1);
	@brr = split(/\./, $s2);
	pop @arr;
	pop @brr;
	$reads1 = join(".", @arr);
	$reads2 = join(".", @brr);
	
	system("date");
	system("echo convert_c2t_or_g2a");
	system("perl $perldir/convert_c2t_or_g2a.pl -i $r1 -p p1 -o $outdir/ProcessedReads/$reads1\.fastq");
	
	system("date");
	system("echo convert_c2t_or_g2a");
	system("perl $perldir/convert_c2t_or_g2a.pl -i $r2 -p p2 -o $outdir/ProcessedReads/$reads2\.fastq");
	
	system("date");
	system("echo hisat2");
	if (defined $ph) {
		system("$ph/hisat2 --phred$q --known-splicesite-infile $refout/splicesites.txt -x $refout/ref_all_C-T_G-A --reorder$opt2 -1 $outdir/ProcessedReads/$reads1\.fastq -2 $outdir/ProcessedReads/$reads2\.fastq -S $outdir/Map/raw_mapping.sam");
	} else {
		system("hisat2 --phred$q --known-splicesite-infile $refout/splicesites.txt -x $refout/ref_all_C-T_G-A --reorder$opt2 -1 $outdir/ProcessedReads/$reads1\.fastq -2 $outdir/ProcessedReads/$reads2\.fastq -S $outdir/Map/raw_mapping.sam");
	}
	system("date");
	system("echo mcPosOnRef");
	system("perl $perldir/mcPosOnRef.pl -sam $outdir/Map/raw_mapping.sam -r1 $r1 -r2 $r2 -o1 $outdir/Map/mapping.sam -o2 $outdir/Filter/filter_mapping.sam");
	
} elsif (exists $opts{"reads1"}) {
	
	$r1 = $opts{"reads1"};
	$s1 = basename($r1);
	@arr = split(/\./, $s1);
	pop @arr;
	$reads1 = join(".", @arr);
	
	system("date");
	system("echo convert_c2t_or_g2a");
	system("perl $perldir/convert_c2t_or_g2a.pl -i $r1 -p p1 -o $outdir/ProcessedReads/$reads1\.fastq");
	
	system("date");
	system("echo hisat2");
	if (defined $ph) {
		system("$ph/hisat2 --phred$q --known-splicesite-infile $refout/splicesites.txt -x $refout/ref_all_C-T_G-A --reorder$opt2 -U $outdir/ProcessedReads/$reads1\.fastq -S $outdir/Map/raw_mapping.sam");
	} else {
		system("hisat2 --phred$q --known-splicesite-infile $refout/splicesites.txt -x $refout/ref_all_C-T_G-A --reorder$opt2 -U $outdir/ProcessedReads/$reads1\.fastq -S $outdir/Map/raw_mapping.sam");
	}
	system("date");
	system("echo mcPosOnRef_sin");
	system("perl $perldir/mcPosOnRef_sin.pl -sam $outdir/Map/raw_mapping.sam -r $r1 -p p1 -o1 $outdir/Map/mapping.sam -o2 $outdir/Filter/filter_mapping.sam");
	
} elsif (exists $opts{"reads2"}) {
	
	$r2 = $opts{"reads2"};
	$s2 = basename($r2);
	@brr = split(/\./, $s2);
	pop @brr;
	$reads2 = join(".", @brr);
	
	system("date");
	system("echo convert_c2t_or_g2a");
	system("perl $perldir/convert_c2t_or_g2a.pl -i $r2 -p p2 -o $outdir/ProcessedReads/$reads2\.fastq");
	
	system("date");
	system("echo hisat2");
	if (defined $ph) {
		system("$ph/hisat2 --phred$q --known-splicesite-infile $refout/splicesites.txt -x $refout/ref_all_C-T_G-A --reorder$opt2 -U $outdir/ProcessedReads/$reads2\.fastq -S $outdir/Map/raw_mapping.sam");
	}else {
		system("hisat2 --phred$q --known-splicesite-infile $refout/splicesites.txt -x $refout/ref_all_C-T_G-A --reorder$opt2 -U $outdir/ProcessedReads/$reads2\.fastq -S $outdir/Map/raw_mapping.sam");
	}
		
	system("date");
	system("echo mcPosOnRef_sin");
	system("perl $perldir/mcPosOnRef_sin.pl -sam $outdir/Map/raw_mapping.sam -r $r2 -p p2 -o1 $outdir/Map/mapping.sam -o2 $outdir/Filter/filter_mapping.sam");
	
} else {
	
	die "Please input the reads file!!!\n";
	
}

###########################################################################################################

system("date");
system("echo watson_and_crick_strand");
system("perl $perldir/watson_and_crick_strand.pl -i $outdir/Filter/filter_mapping.sam -o1 $outdir/Filter/filter_mapping_watson.sam -o2 $outdir/Filter/filter_mapping_crick.sam");

system("date");
system("echo samtools_mclev_by_chr");
if (exists $opts{"pathToSamtools"}){
	my $ps = $opts{"pathToSamtools"};
	system("perl $perldir/samtools_mclev_by_chr.pl -i $outdir/Filter/filter_mapping_watson.sam -t watson -ps $ps -perldir $perldir -refi $refin -refo $refout -o $outdir");
	system("perl $perldir/samtools_mclev_by_chr.pl -i $outdir/Filter/filter_mapping_crick.sam -t crick -ps $ps -perldir $perldir -refi $refin -refo $refout -o $outdir");
} else {
	system("perl $perldir/samtools_mclev_by_chr.pl -i $outdir/Filter/filter_mapping_watson.sam -t watson -perldir $perldir -refi $refin -refo $refout -o $outdir");
	system("perl $perldir/samtools_mclev_by_chr.pl -i $outdir/Filter/filter_mapping_crick.sam -t crick -perldir $perldir -refi $refin -refo $refout -o $outdir");
}

system("date");
system("echo subjobs watson strand");
system("perl $perldir/subjobs.pl -i $outdir/Samtools/JOB_LIST_watson.txt");

sleep 10;

system("date");
system("echo subjobs crick strand");
system("perl $perldir/subjobs.pl -i $outdir/Samtools/JOB_LIST_crick.txt");

system("date");
system("echo waiting time");
sleep 120;

my $chr_num = 0;
my $samm;
my $i = 0;

opendir INdir, "$outdir/Samtools/scripts";
while ($samm = readdir INdir) {
	if ($samm =~ /\.sh$/) {
		$chr_num ++;
	}
}
close Indir;

while (1) {
	$i = 0;
	opendir INdir, "$outdir/Samtools/scripts";
	while ($samm = readdir INdir) {
		if ($samm =~ /\.finish$/) {
			$i ++;
		}
	}
	close Indir;
	last if $i == $chr_num;
	sleep 120;
}

system("date");
system("echo temp files are removing");
system("rm -r $outdir/ProcessedReads $outdir/Samtools");
system("rm $outdir/Map/raw_mapping.sam $outdir/Map/doubled_gene.gtf $outdir/Filter/filter_mapping_*.sam");
unless (exists $opts{'convertRef'}) {
	system("rm -r $refout/chr*.fa");
}
system("date");
system("echo completed!!!");
